C J. R. Yang, National Taiwan University, Graduate School of Engineering
C H. K. D. H. Bhadeshia, University of Cambridge,
C Department of Materials Science and Metallurgy,
C Pembroke Street, Cambridge CB2 3QZ
C
C 31 March 1987
C
C Calculation of the one-dimensional thickening rate constant for the
C diffusion controlled growth of austenite from a mixture of
C bainitic ferrite and austenite. Assumes lack of soft impingement.
C The austenite and ferrite are both semi-infinite in extent.
C
C 27 September 1985 same as .CONCDIF:RATE, BUT modified to
C allow easier low T calculations and provide iteration error messages
C TYPICAL DATASET
C 1 (The number of alloys included in the analysis)
C 5 (number of temperature/concentration pairs to follow)
C 0.55 0.49 1.16 0.0 0.0 0.0 0.0
C (C  Si  Mn  Ni  Mo  Cr  V wt.%)
C 780 0.0100
C (Temperature in centigrade, equilibrium mole fraction of C in austenite)
C 740 0.0176
C 700 0.0266
C 660 0.0389
C 600 0.0552
C Modified 24-6-85 to include XINCR
C- this limits no. of Diff coeffs calculated for T<700C
C
C
C XONE is the starting mole fraction of carbon in austenite
C XGAG, XAGA are the paraequilibrium mole fractions of carbon in
C  austenite and in ferrite respectively.
C
C     DIFF =DIFFUSIVITY OF CARBON IN AUSTENITE
C     Z    =COORDINATION OF INTERSTIAL SITE
C     PSI  =COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT
C     THETA=NO. C ATOMS/ NO. FE ATOMS
C     ACTIV=ACTIVITY OF CARBON IN AUSTENITE
C     X    =MOLE FRACTION OF CARBON
C     T    =ABSOLUTE TEMPERATURE
C     SIGMA=SITE EXCLUSION PROBABLITY
C     W    =CARBON CARBON INTERACTION ENERGY IN AUSTENITE
C
      IMPLICIT REAL*8(A-H,K-Y), INTEGER(I,J,Z)
      DOUBLE PRECISION DIFF(500),TEMPC(40,10),ALP(40,10),CARB(500)
      DOUBLE PRECISION C(8)
C
C HH = Planck constant (Js).
C KK = Boltzman constant (J/K).
C R  = Universal gas constant (J/mol/K)
C
      HH = 6.6260755D-34
      KK = 1.380658D-23
      R  = 8.31451D0
      WRITE(*,*)
      WRITE(*,*) 'Input number of different steel compositions:'
      READ (*,*) I6
      DO 190 III=1,I6
         WRITE (*,*) 'Input  C,  Si,  Mn,  Ni,  Mo,  Cr,  V wt%:'
         READ (5,*) C(1),C(2),C(3),C(4),C(5),C(6),C(7)
         WRITE(*,*) 'Input number of different temperatures:'
         READ (*,*) I8
         IJ = 0
         WRITE(*,1)
         WRITE(*,2) C(1),C(2),C(3),C(4),C(5),C(6),C(7)
         CALL MAP_STEEL_OMEGA(C,W,XONE,T10,T20,IJ)
         WRITE(*,3) C(1),C(2),C(3),C(4),C(5),C(6),C(7)
         WRITE(*,4) W,XONE
         Z  = 12
         A5 = 1.0D0
         RADIUS = 0D0
         VMAX = 0D0
         M1   = 0D0
         DO 170 II22=1,I8
            WRITE(*,*)
            WRITE(*,*) 
     &      'Input temperature (deg.C) and equilibrium C mole fraction:'
            READ (*,*) T,XGAG
            IF (XGAG .GE. XONE) THEN
               WRITE(*,*)
               WRITE(*,*)
     &         '>>>> Equilibrium C concentration greater than starting',
     &                   ' conc. in austenite.'
               WRITE(*,*) '>>>> Growth of austenite not possible.'
               GOTO 180
            ENDIF
            CTEMP = T
            T     = T+273.00D0
            II2   = 0
            XALPHA = MAP_STEEL_XALPH(T)
            WRITE(*,5) T,CTEMP,XGAG,XALPHA
            DASH = (KK*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
            DO 150 II=1,1000
               CARB(1) = XONE
               IF (II .GT. 1) GOTO 100
               GOTO 140
  100          IF (CTEMP .LT. 700.0) GOTO 110
               GOTO 120
  110          XINCR = 0.0001D0
               GOTO 130
  120          XINCR = 0.001D0
  130          CARB(II) = CARB(II-1)-XINCR
               IF (CARB(II) .LT. XGAG) GOTO 160
  140          X   = CARB(II)
               II2 = II2+1
               THETA = X/(A5-X)
               ACTIV = MAP_STEEL_CG(X,T,W,R)
               ACTIV = DEXP(ACTIV)
               DACTIV = MAP_STEEL_DCG(X,T,W,R)
               DACTIV = DACTIV*ACTIV
               DACTIV = DACTIV*A5/((A5+THETA)**2)
               SIGMA  = A5-DEXP((-(W))/(R*T))
               PSI = ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA +
     &               (Z/2)*(A5+Z/2)*(A5-SIGMA)*THETA*THETA))) +
     &               (A5+THETA)*DACTIV
               DIFF(II) = DASH*PSI
  150       CONTINUE
  160       II3 = 0
C
C  Alternative NAG routine: CALL D01GAF(CARB,DIFF,II2,ANS,ERROR,II3) 
C
            CALL MAP_UTIL_TRAPE(CARB,DIFF,ANS,II2)
            ANS = ANS/(XGAG-XONE)
            CALL AL(XGAG,XONE,XALPHA,ANS,CTEMP,ALPHA)
            I7 = I8+1-II22
            ALP(III,I7)   = ALPHA
            TEMPC(III,I7) = CTEMP
            WRITE(*,6) ALPHA
  170    CONTINUE
  180    WRITE(*,7)
  190 CONTINUE
      WRITE(*,8)
      DO 210 I=1,I6
         DO 200 J=1,I8
            IF (TEMPC(I,J) .LT. 1.0D-10) GOTO 200
            XXX = 1.0D0/(ALP(I,J)**2)
            WRITE (*,9) I,TEMPC(I,J),ALP(I,J),XXX
  200    CONTINUE
  210 CONTINUE
      STOP
    1 FORMAT(//'****************************************************'
     &         '**********************'//
     &         ' Element:       C      Si      Mn      Ni      Mo'
     &         '      Cr      V')
    2 FORMAT(  ' conc. wt%: ',7(F7.4,X))
    3 FORMAT(  ' mole frac: ',7(F7.4,X)/)
    4 FORMAT(' Carbon-carbon interaction energy in austenite = ',F7.1,
     &       ' J/mol'/
     &       ' Starting mole fraction of carbon in austenite = ',F7.4//)
    5 FORMAT(' Temperature = ',F8.2,' K   or ',F8.2,' deg. C.'/
     &       ' Equilibrium mole fraction of C in austenite   = ',F8.5/
     &       ' Equilibrium mole fraction of C in ferrite     = ',F8.5)
    6 FORMAT(' Parabolic rate constant (cm/s**0.5)           = ',D11.4)
    7 FORMAT(//'****************************************************'
     &         '**********************')
    8 FORMAT(//'  Alloy  Temperature  Parabolic rate constant  '
     &         '1/(p.r.c.)**2'/
     &    '           (deg.C)          (cm/s**0.5)          (s/cm/cm)'/)
    9 FORMAT(3X,I2,5X,F7.2,11X,D11.4,8X,D11.4)
      END
C*******************************************************************************
C CG CG DOUBLE PRECISION FUNCTION GIVING LFG  LN(ACTIVITY) OF CARBON IN
C AUSTENITE
      DOUBLE PRECISION FUNCTION MAP_STEEL_CG(X,T,W,R)
      IMPLICIT NONE
      DOUBLE PRECISION AJ,DG,EG,T,R,W,X
      AJ=1.0-DEXP(-W/(R*T))
         IF(X .LE. 1.0D-10) THEN
           MAP_STEEL_CG=DLOG(1.0D-10)
         ELSE
           DG=DSQRT(1.0-2.0*(1.0+2.0*AJ)*X+(1.0+8.0*AJ)*X*X)
           EG=5.0*DLOG((1.0-2.0*X)/X)+6.0*W/(R*T)+
     &     (38575.0-13.48*T)/(R*T)
           MAP_STEEL_CG=EG+6.0D+00*DLOG((DG-1+3*X)/(DG+1-3*X))
         ENDIF
      RETURN
      END
C*******************************************************************************
C DCG DCG
C FUNCTION GIVING DIFFERENTIAL OF LN(ACTIVITY) OF CARBON IN AUSTENITE, LFG
C DIFFERENTIAL IS WITH RESPECT TO X
      DOUBLE PRECISION FUNCTION MAP_STEEL_DCG(X,T,W,R)
      IMPLICIT NONE
      DOUBLE PRECISION AJ,DG,DDG,X,T,W,R
      AJ=1.0-DEXP(-W/(R*T))
      DG=DSQRT(1.0-2.0*(1.0+2.0*AJ)*X+(1.0+8.0*AJ)*X*X)
      DDG=(0.5/DG)*(-2.0-4.0*AJ+2.0*X+16.0*AJ*X)
      MAP_STEEL_DCG=-((10.0/(1.0-2.0*X))+(5.0/X))+6.0*((DDG+3.0)
     &/(DG-1.0+3.0*X)-(DDG-3.0)/(DG+1.0-3.0*X))
      RETURN
      END
C*******************************************************************************
      SUBROUTINE MAP_STEEL_OMEGA(C,W,XBAR,T10,T20,J)
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
      IMPLICIT NONE
      DOUBLE PRECISION C(8),W,P(8),B1,B2,Y(8),T10,T20,B3,XBAR
      INTEGER IU,J
      B3=0.0D+00
      IF(J .EQ. 1) GOTO 2
      C(8)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)
      C(8)=100.0D+00-C(8)
      C(8)=C(8)/55.84D+00
      C(1)=C(1)/12.0115D+00
      C(2)=C(2)/28.09D+00
      C(3)=C(3)/54.94D+00
      C(4)=C(4)/58.71D+00
      C(5)=C(5)/95.94D+00
      C(6)=C(6)/52.0D+00
      C(7)=C(7)/50.94D+00
2     B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      DO 107 IU=2,7
      Y(IU)=C(IU)/C(8)
107   CONTINUE
      DO 106 IU=1,8
      C(IU)=C(IU)/B1
106   CONTINUE
      XBAR=C(1)
      XBAR=DINT(10000.0D+00*XBAR)
      XBAR=XBAR/10000
      B2=0.0D+00
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
      T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-44*Y(7)
C
C     Polynomials representing carbon-carbon interaction energy (J/mol)
C     in austenite as a function of the molecular fraction of individual solutes.
C
      P(2)=-2.4233D+07+6.9547D+07*C(2)
      P(2)=+3.864D+06+P(2)*C(2)
      P(2)=+45802.87+(-280061.63+P(2)*C(2))*C(2) 
      P(2)=2013.0341+(763.8167+P(2)*C(2))*C(2)
      P(3)=2.0119D+06+(3.1716D+07-1.3885D+08*C(3))*C(3)
      P(3)=+6287.52+(-21647.96+P(3)*C(3))*C(3)
      P(3)=2012.067+(-1764.095+P(3)*C(3))*C(3)
      P(4)=-2.4968D+07+(1.8838D+08-5.5531D+08*C(4))*C(4)
      P(4)=-54915.32+(1.6216D+06+P(4)*C(4))*C(4)
      P(4)=2006.8017+(2330.2424+P(4)*C(4))*C(4)
      P(5)=-1.3306D+07+(8.411D+07-2.0826D+08*C(5))*C(5)
      P(5)=-37906.61+(1.0328D+06+P(5)*C(5))*C(5)
      P(5)=2006.834+(-2997.314+P(5)*C(5))*C(5)
      P(6)=+8.5676D+06+(-6.7482D+07+2.0837D+08*C(6))*C(6)
      P(6)=+33657.8+(-566827.83+P(6)*C(6))*C(6)
      P(6)=2012.367+(-9224.2655+P(6)*C(6))*C(6)
      P(7)=+5411.7566
     &+(250118.1085-4.1676D+06*C(7))*C(7)
      P(7)=2011.9996+(-6247.9118+P(7)*C(7))*C(7)
      DO 108 IU=2,7
      B3=B3+P(IU)*Y(IU)
      B2=B2+Y(IU)
108   CONTINUE
      IF (B2 .EQ. 0.0D+00) GOTO 455
      W=(B3/B2)*4.187
      GOTO 456
455   W=8054.0
456   CONTINUE
      RETURN
      END
C ******************************************************************************
C Function giving the equilibrium mole fraction of carbon in ferrite
C based on my paper on first order quasichemical theory, Metal Science
C
      DOUBLE PRECISION FUNCTION MAP_STEEL_XALPH(T)
C
      IMPLICIT NONE
      DOUBLE PRECISION T,CTEMP,T0
      PARAMETER(T0=273.0D0)
C
      CTEMP = (T-T0)/900.0D+00
      MAP_STEEL_XALPH=0.1528D-02-0.8816D-02*CTEMP+0.2450D-01*CTEMP**2
     &-0.2417D-01*CTEMP**3+
     &0.6966D-02*CTEMP**4
C
      RETURN
      END
C*******************************************************************************
      SUBROUTINE AL(XGAG,XONE,XAGA,DIFF,CTEMP,ALPHA)
C
      IMPLICIT NONE
      DOUBLE PRECISION DIFF,ALPHA,XAGA,XGAG,XONE,CTEMP
      DOUBLE PRECISION ALPH,DER,DUMMY1,DUMMY2,FUN2
      INTEGER III
C
      III    = 0
      ALPHA  = 1.8D-06
C
C  GUESS A VALUE OF ALPHA:
C
      DUMMY2 = ALPHA
  100 CALL FUNN(DUMMY1,ALPHA,XGAG,XONE,XAGA,DIFF)
      III = III+1
      IF (III .GT. 10) GOTO 120
      IF (DABS(DUMMY1) .GT. 0.0001D-06) GOTO 110
      GOTO 120
  110 DUMMY2 = DUMMY1
      ALPH   = ALPHA*1.0000001D+00
      CALL FUNN(FUN2,ALPH,XGAG,XONE,XAGA,DIFF)
      DER   = (DUMMY1-FUN2)/(ALPHA-ALPH)
      ALPHA = ALPHA-DUMMY1/DER
      GOTO 100
  120 CONTINUE
      RETURN
      END
C*************************************************************************
C 'FUN' COMPUTES EQ.2 OF KINSMAN AND AARONSONS TRANSFORMATION AND
C HARDENABILITY PAPER.  DIFF=INTEGRATED AVERAGE DIFFUSIVITY OF C IN
C GAMMA, ALPHA = PARABOLIC RATE CONSTANT.
C
      SUBROUTINE FUNN(FUN,ALPHA,XGAG,XONE,XAGA,DIFF)
C
      IMPLICIT NONE
      DOUBLE PRECISION ALPHA,DIFF,FUN,XAGA,XGAG,XONE
C
      FUN = 2D0*(XONE-XGAG)/(XONE-XAGA)
      FUN = FUN*DSQRT(DIFF/3.14159D0)
      FUN = FUN-(ALPHA*DEXP(ALPHA*ALPHA/(4.0D0*DIFF))*(1.0D0 -
     &      DERF(ALPHA/(2.0D0*DSQRT(DIFF)))))
      RETURN
      END
C***********************************************************************************
C        SUBROUTINE TRAPE
C
C        PURPOSE
C           TO COMPUTE THE VECTOR OF INTEGRAL VALUES FOR A GIVEN
C           GENERAL TABLE OF ARGUMENT AND FUNCTION VALUES.
C
C        USAGE
C             CALL TRAPE (X,Y,Z,NDIM)
C
C        DESCRIPTION OF PARAMETERS
C           X      - DOUBLE PRECISION INPUT VECTOR OF ARGUMENT VALUES.
C           Y      - DOUBLE PRECISION INPUT VECTOR OF FUNCTION VALUES.
C           Z      - THE RESULTING DP. VECTOR OF INTEGRAL VALUES. Z MAY BE
C                    IDENTICAL WITH X OR Y.
C           NDIM   - THE DIMENSION OF VECTORS X,Y,Z. NDIM MAX. 1000
C
C        REMARKS
C           NO ACTION IN CASE NDIM LESS THAN 1.
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           BEGINNING WITH Z(1)=0, EVALUATION OF VECTOR Z IS DONE BY
C           MEANS OF TRAPEZOIDAL RULE (SECOND ORDER FORMULA).
C           FOR REFERENCE, SEE
C           F.B.HILDEBRAND, INTRODUCTION TO NUMERICAL ANALYSIS,
C           MCGRAW-HILL, NEW YORK/TORONTO/LONDON, 1956, PP.75.
C
C       Subroutine stolen from SSP (IBM), cf. PLUS/1000 X019
C           HO    <881109.1345>
C     ..................................................................
C
      SUBROUTINE MAP_UTIL_TRAPE(X,Y,ANS,NDIM)
C
C
      IMPLICIT NONE
      DOUBLE PRECISION X(1000),Y(1000),AZ(1000)
      DOUBLE PRECISION SUM1,SUM2,ANS
      INTEGER NDIM,I
C
      SUM2=0.D+00
      IF(NDIM-1)4,3,1
C
C     INTEGRATION LOOP
    1 DO 2 I=2,NDIM
      SUM1=SUM2
      SUM2=SUM2+.5D+00*(X(I)-X(I-1))*(Y(I)+Y(I-1))
    2 AZ(I-1)=SUM1
    3 AZ(NDIM)=SUM2
      ANS=SUM2
    4 RETURN
      END
